<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;
use App\User;
use Illuminate\Support\Facades\Validator;
use Laravel\Socialite;
use GuzzleHttp;

class AuthController extends Controller
{
    /**
     * Create user
     *
     * @param  string name
     * @param  string email
     * @param  string password
     * @param  string password_confirmation
     * @return JSON
     */

    protected function validators(array $data)
    {
        return Validator::make($data, [
            'first_name' => 'required|string',
            'last_name' => 'required|string',
            'email' => 'required|string|email|unique:users',
            'password' => 'required|string|confirmed',
            'dob' => 'required|date_format:Y-m-d',
            'is_admin' => 'required|boolean',

        ]);
    }
    public function register(Request $request)
    {

        $validator = $this->validators($request->all());

        if ($validator->fails()) {
            return response()->json($validator->messages(), 200);
        }

        $dob = (@$request->b_day && @$request->b_month  && @$request->b_year) ? ($request->b_year."-". @$request->b_month."-".@$request->b_day) : null;
        $date_of_birth = (!empty($dob)) ? date("Y-m-d",  strtotime($dob))  : null;        

        $user = new User([
            'name_suffix' => $request->name_suffix,
            'first_name' => $request->first_name,
            'last_name' => $request->last_name,
            'email' => $request->email,
            'fb_user_id' => $request->fb_id,
            'google_user_id' => $request->google_user_id,
            'password' => bcrypt($request->password),
            'is_loyalty_program' => (@$request->is_loyalty_program) ? '1' : '0',
            'is_mysushi_offers' => (@$request->is_mysushi_offers) ? '1' : '0',
            'phone' => (!empty($request->phone)) ? $request->phone : null,
            'dob' => $date_of_birth
        ]);
        $user->save();

        return response()->json([
            'message' => 'Successfully created user!',
        ], 201);
    }

    /**
     * Login user and create token
     *
     * @param  string email
     * @param  string password
     * @param  boolean remember_me
     * @return JSON
     */

    public function googleAuth(Request $request){
        $check = User::where('google_user_id', $request->google_user_id)->orWhere('email', $request->email)->first();
        $user_id = "";
        if(@$check->google_user_id == ""){
            $user = new User;
            $user::where('email', $request->email)
                ->update([
                    'google_user_id' => $request->google_user_id
             ]);
        }
        if (!@$check->id) {

            $user = new User;
            $user->google_user_id = $request->google_user_id;
            $user->first_name = $request->first_name;
            $user->last_name = $request->last_name;
            $user->email = $request->email;
            $user->password =  bcrypt('mysushi@12345') ; //by default pass set to mysushi@12345
            $user->save();
            $user_id = $user->id;

        }
        $user_id =  (!@$check->id) ? $user_id : $check->id;

        Auth::loginUsingId($user_id);
        $user = $request->user();
        $tokenResult = $user->createToken('MySushi');
        $token = $tokenResult->token;

        if ($request->remember_me) {
            $token->expires_at = Carbon::now()->addWeeks(1);
        }
        $token->save();

        return response()->json([
            'access_token' => $tokenResult->accessToken,
            'token_type' => 'Bearer',
            'expires_at' => Carbon::parse(
                $tokenResult->token->expires_at
            )->toDateTimeString(),
            'message' => 'Authenticated.',
            'user' => $user,
            'code' => '200'
        ]);
    }
    public function fbAuth(Request $request){


        $check = User::where('fb_user_id', $request->id)->orWhere('email', $request->email)->first();
        $user_id = "";
        if(@$check->fb_user_id == ""){
            $user = new User;
            $user::where('email', $request->email)
                ->update([
                    'fb_user_id' => $request->id
                ]);
        }
        if (!@$check->id) {

            $make_dummy_email = "user".$request->id."@mysushi.ph";
            $user = new User;
            $user->fb_user_id = $request->id;
            $user->first_name = $request->first_name;
            $user->last_name = $request->last_name;
            $user->email = ($request->email != "" || $request->email != null) ? $request->email : $make_dummy_email;
            $user->password =  bcrypt('mysushi@12345') ; //by default pass set to mysushi@12345
            $user->save();
            $user_id = $user->id;

        }


        $user_id =  (!@$check->id) ? $user_id : $check->id;

        Auth::loginUsingId($user_id);

        $user = $request->user();
        $tokenResult = $user->createToken('MySushi');
        $token = $tokenResult->token;

        if ($request->remember_me) {
            $token->expires_at = Carbon::now()->addWeeks(1);
        }
        $token->save();

        return response()->json([
            'access_token' => $tokenResult->accessToken,
            'token_type' => 'Bearer',
            'expires_at' => Carbon::parse(
                $tokenResult->token->expires_at
            )->toDateTimeString(),
            'message' => 'Authenticated.',
            'user' => $user,
            'code' => '200'
        ]);
    }
    public function login(Request $request)
    {
        $request->validate([
            'email' => 'required|string|email',
            'password' => 'required|string',
            'remember_me' => 'boolean'
        ]);

        $credentials = request(['email', 'password']);
        if (!Auth::attempt($credentials)) {
            return response()->json([
                'message' => 'E-mail or Password is incorrect!'
            ], 422);
        }

        $user = $request->user();
        $tokenResult = $user->createToken('MySushi');
        $token = $tokenResult->token;

        if ($request->remember_me) {
            $token->expires_at = Carbon::now()->addWeeks(1);
        }
        $token->save();

        return response()->json([
            'access_token' => $tokenResult->accessToken,
            'token_type' => 'Bearer',
            'expires_at' => Carbon::parse(
                $tokenResult->token->expires_at
            )->toDateTimeString(),
            'message' => 'Authenticated.',
            'user' => $user,
            'code' => '200'
        ]);
    }

    /**
     * Logout user (Revoke the token)
     *
     * @return JSON
     */
    public function logout(Request $request)
    {
        $request->user()->token()->revoke();
        return response()->json([
            'message' => 'Successfully logged out'
        ]);
    }

    /**
     * Get the authenticated User
     *
     * @return JSON user object
     */
    public function user(Request $request)
    {
        return response()->json($request->user());
    }
}
