<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\OrdersStatusRequest;
use App\Http\Resources\OrdersResource;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Http;
use Carbon\Carbon;
use App\Models\Order;
use App\Models\OrderItem;
use App\Models\Paypal;
use App\Models\UserPoint;
use App\User;
use App\LoyaltyProgram;
use Auth;
use Mail;
use GuzzleHttp\Client;

class OrdersController extends Controller
{
  protected $client;
  public function __construct(){
    $this->client = new ClientsController();
    parent::__construct();
  }
    /**
     * Get orders with pagination
     *
     * @return  OrdersResource  orders with pagination data
     */
    public function index()
    {
        if (!empty($_GET['date'])) {
            if ($_GET['date'] == 'asc' || $_GET['date'] == 'desc') {
                $clients = Order::with(['orderItems' => function ($query) {
                    $query->with(['product'])->get();
                }, 'orderClient'])->orderBy('created_at', $_GET['date'])->paginate(50);

                return OrdersResource::collection($clients);
            }
        }

        if (!empty($_GET['date_receive'])) {
            if ($_GET['date_receive'] == 'asc' || $_GET['date_receive'] == 'desc') {
                $clients = Order::with(['orderItems' => function ($query) {
                    $query->with(['product'])->get();
                }, 'orderClient'])->orderBy('date_receive', $_GET['date_receive'])->paginate(50);

                return OrdersResource::collection($clients);
            }
        }

        if (!empty($_GET['payment_status'])) {
            if ($_GET['payment_status'] == 'asc' || $_GET['payment_status'] == 'desc') {
                $clients = Order::with(['orderItems' => function ($query) {
                    $query->with(['product'])->get();
                }, 'orderClient'])->orderBy('payment_status', $_GET['payment_status'])->paginate(50);

                return OrdersResource::collection($clients);
            }
        }

        if (!empty($_GET['payment_type'])) {
            if ($_GET['payment_type'] == 'asc' || $_GET['payment_type'] == 'desc') {
                $clients = Order::with(['orderItems' => function ($query) {
                    $query->with(['product'])->get();
                }, 'orderClient'])->orderBy('payment_type', $_GET['payment_type'])->paginate(50);

                return OrdersResource::collection($clients);
            }
        }

        $orders = Order::with(['orderItems' => function ($query) {
            $query->with(['product'])->get();
        }, 'orderClient'])->paginate(50);

        return OrdersResource::collection($orders);
    }

    /**
     * Get order details by ID
     *
     * @param   int  $id  order ID
     *
     * @return  JSON      order data or message
     */
    public function show($id)
    {
        try {
            $orders = Order::with(['orderItems' => function ($q) use ($id) {
                $q->with(['product' => function ($q1) use ($q, $id) {
                    $q1->with(['promoItem' => function ($q2) use ($q1, $q, $id) {
                        $q2->with(['promo'])->get();
                    }]);
                }])->get();
            }, 'orderClient', 'orderLoyaltyoffer'])->findOrFail($id);

            return response()->json([
                'data' => $orders,
            ]);
        } catch (Exception $e) {
            return response()->json([
                'message' => 'Order data not found.'
            ], 404);
        }
    }

    /**
     * Change order status
     *
     * @param   OrdersStatusRequest  $request  [$request description]
     *
     * @return  [type]                         [return description]
     */
    public function status(OrdersStatusRequest $request)
    {
        try {
            $id = $request->order_id;
            $status = $request->order_payment_status;

            $order = Order::with(['orderItems' => function ($query) {
                $query->with(['product'])->get();
            }, 'orderClient'])->findOrFail($id);

            if ($order->payment_type == 'cod' && $status == 'completed') {
                $order->payment_status = $status;
                $order->date_receive = Carbon::now();
            } else {
                //farid code for delete userpoints if order is cancelled
                if ($status == 'cancelled') 
                {
                    UserPoint::where('order_id', $id)->delete();
                    $usertotalpoints = UserPoint::where('client_id', $order->client_id)->whereBetween('created_at', [Carbon::now()->subYear(1), date('Y-m-d H:i:s')])->sum('points');
                    //dd($usertotalpoints); 
                    if($usertotalpoints != 0)
                    {
                        $selectedprogram = LoyaltyProgram::Select('id')->where([['min_points','<=',$usertotalpoints],
                                                 ['max_points','>',$usertotalpoints]])->first();
                        $user                           =   User::find($order->client_id);
                        $user->loyality_program_level   =   $selectedprogram->id;
                        //remove offer from users table
                        if(!empty($user->avail_mysushi_offers))
                        {
                            $availoffersusertable = explode(',', $user->avail_mysushi_offers);
                            while(($i = array_search($order->user_avail_offer, $availoffersusertable)) !== false) {
                                unset($availoffersusertable[$i]);
                            }
                            $availoffersusertable  =   implode(',', $availoffersusertable);
                            $user->avail_mysushi_offers     =  $availoffersusertable;
                        }
                        $user->save(); 
                    }
                    else
                    {
                        $user                           =   User::find($order->client_id);
                        $user->loyality_program_level   =   0;
                        $user->save(); 
                    }
                    //$order->user_avail_offer = Null;  
                }
                //farid code ends
                $order->payment_status = $status;
            }
            if ($order->save()) {
                DB::commit();

                return response()->json([
                    'data' => $order,
                ]);
            } else {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => 'Error while changing order status to database.'
                ], 500);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (Exception $e) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    /**
     * Change order status
     *
     * @param   OrdersStatusRequest  $request  [$request description]
     *
     * @return  [type]                         [return description]
     */
    public function saveorder(Request $request)
    {
//        dd($request->all());
        $orderID = $status = '';
        $cartArray = json_decode($request->cartArray);
        $validAddress = json_decode($request->validAddress);
        $user = $request->user;
        $totalAmount = $request->totalAmount;
        $loyalityOffers = $request->loyality_offers;
        // $activePayment = $request->activePayment;
        // $usertotalpoints = UserPoint::where('client_id', '1')->whereBetween('created_at', [Carbon::now()->subYear(1), date('Y-m-d H:i:s')])->sum('points');
        // $selectedprogram = LoyaltyProgram::Select('id')->where([['min_points','<',$usertotalpoints],
        //                                          ['max_points','>',$usertotalpoints]])->first();
        // echo $selectedprogram->id;die;
        switch ($request->activePayment) {
            case 0:
                $activePayment = 'credit_card';
                break;
            case 1:
                $activePayment = 'paypal';
                break;
            case 2:
                $activePayment = 'cod';
                break;
        }

        $order = new Order();
        $order->order_unique_id = date("YmdHis");
        $order->total_amount =  $request->finalTotal; //this is a final total amount
        $order->delivery_charge = $request->delivery_fee;
        $order->payment_type = $activePayment;
        $order->sub_total = $totalAmount; //this is a sub total amount
        $order->discount_in_percent = number_format($request->discountInPercent);
        $order->promo_code = $request->promoCode;
        $order->payment_status = "new";
        $order->notes = "";
        $order->client_id = $user;
        if($loyalityOffers != "0")
        {
            $order->user_avail_offer = $loyalityOffers;
        }
        $order->address1 = $request->address1;
        $order->address2 = $request->address2;
        $order->building_and_unit_number = $request->building_and_unit_number;
        $order->delivery_address = $request->delivery_address;
        $order->source = $request->source;
        $order->is_delivery_now = ($request->isDeliveryNow == "0") ? 1 : 0;
        $order->date_deliver = date('Y-m-d', strtotime($request->date_deliver));
        $order->time_deliver =  date('H:i:s', strtotime($request->time_deliver));
        $order->date_receive = Carbon::now();
        $order->created_at = Carbon::now();
        $order->updated_at = Carbon::now();
        $order->is_paid = 0;

        // $promo = Promo::with('promoCategories')->where('code', $request->promoCode)->get();
        // dd($promo);
/* ------- TALHA CODE STARTS HERE ---------- */
        if ($order->save()){

//            dd($order);
            $orderID = $order->id;
            $status = 200;

            foreach ($cartArray as $cart) {
                $selectedAllergies = array();
                $orderItem = new OrderItem();

                $orderItem->product_item_id = $cart->id;
                $orderItem->product_item_count = $cart->qty;
                if (!empty($cart->allergies)) {
                    foreach ($cart->allergies as $allergy) {
                        if ($allergy->selected) {
                            array_push($selectedAllergies, $allergy->name);
                        }
                    }
                }

                if (!empty($selectedAllergies)){
                    $orderItem->allergies = implode(',', $selectedAllergies);
                }else{
                    $orderItem->allergies = null;
                }


                $orderItem->order_id =  $order->id;

                $orderItem->save();
            }

            $user = User::find($user);
            /* ------- Farid code starts ---------- */

            if($loyalityOffers != "0")
            {
                $useravailoffers    =   $user->avail_mysushi_offers;
                if(!empty($useravailoffers))
                {
                    $user->avail_mysushi_offers     =   $useravailoffers.','.$loyalityOffers;
                }
                else
                {
                    $user->avail_mysushi_offers     =   $loyalityOffers;
                }
            }

            /* ------- Farid code ends ---------- */
            $user->no_of_orders = (@$user->no_of_orders) ? @$user->no_of_orders + 1 : 1;
            $user->total_spent = $user->total_spent + $order->total_amount;
            $user->address1 = $request->address1;
            $user->phone=$request->phone_number;
            $user->save();
            {
                /* ------- Farid code starts ---------- */

                $userID     = $user->id;
                $userpoint  = new UserPoint();
                $userpoint->client_id   =   $userID;
                $totalpoints            =   $order->total_amount/50;
                $userpoint->points      =   $totalpoints+1;
                $userpoint->order_id    =   $orderID;
                $userpoint->save();
                {

                    $usertotalpoints = UserPoint::where('client_id', $userID)->whereBetween('created_at', [Carbon::now()->subYear(1), date('Y-m-d H:i:s')])->sum('points');
                    $selectedprogram = LoyaltyProgram::Select('id')->where([['min_points','<=',$usertotalpoints],
                                                 ['max_points','>=',$usertotalpoints]])->first();
                    $user1              =   User::find($userID);
                    $user1->loyality_program_level =    $selectedprogram->id;
                    $user1->save();
                    {
                        $this->client->PaypalPaymentLogs(json_encode(['order_id' => $orderID,'status' => $status]) , $orderID , $request->activePayment , 'order_placement:200' , 'save_order');
                    }   

                    
                }
                 /* ------- Farid code ends ---------- */
            }

            //send email template on order placement
            $emailcontent = array(
                'payment_method' => $activePayment,
                'address' => $request->address1.' '.$request->address2.' '.$request->building_and_unit_number,
                'email' => $request->email,
                'order_id' => $order->id,
                'date' => date("F j, Y"),
                'delivery_fee' => $request->delivery_fee,
            );
            $request->email = $user->email;
            Mail::send('email.orderplacement', $emailcontent, function ($m) use ($request)
            {
                $m->from('no-reply@mysushi.ph', 'Mysushi');
                $m->to($request->email)->subject('Order Confirmation!');
            });
//            dd(12);
          
        }else{
            $this->client->PaypalPaymentLogs(json_encode(['order_id' => null,'status' => 500]) , 0 , $request->activePayment , 'placement_failed' , 'save_order');
            $orderID = null;
            $status = 500;
        }

        // Call api to insert record in DFG database starts

        // dd($lastorder);
             $apiarray                                   =   [];
             $apiarray['orders']['order_unique_id']      =   $order->order_unique_id;
             $apiarray['orders']['date_receive']         =   date("Y-m-d");
             $apiarray['orders']['brand']                =   'MySushi';
             $apiarray['orders']['source']               =   $request->source;
             $apiarray['orders']['delivery_fee']         =   $request->delivery_fee;
             $apiarray['orders']['status']               =   'New';
             $apiarray['orders']['total_price']          =   $order->total_amount;
             $apiarray['orders']['payment_type']         =   $order->payment_type;
             $apiarray['orders']['payment_status']       =   $order->payment_status;
             $apiarray['orders']['address1']             =   $order->address1;
             $apiarray['orders']['address2']             =   $order->address2;
             $apiarray['orders']['notes']                =   $order->notes;
             $apiarray['orders']['delivery_address']     =   $order->delivery_address;
             $lastorder_items = DB::table('order_items')->where('order_id', $orderID)->get();

             $i=0;
             foreach ($lastorder_items as $lastorder_item) {
                 $product = DB::table('products')->where('id',$lastorder_item->product_item_id)->first();
//                 $order_ingredients = DB::table('order_item_ingredients')->where('order_id',$orderID)->where('product_item_id',$product->id)
//                 ->get();
//                 $ingredients_name = [];
//                 foreach ($order_ingredients as $order_ingredient) {
//                     $ingredient = DB::table('ingredients')->where('id', $order_ingredient->product_ingredient_id)->first();
//                     $ingredients_name[] = $ingredient->name;
//                 }
//                 $allergies = implode(",",$ingredients_name);
                 $apiarray['order_items'][$i]['product_id']=   $product->id;
                 $apiarray['order_items'][$i]['name']      =   $product->name;
                 $apiarray['order_items'][$i]['quantity']  =   $lastorder_item->product_item_count;
                 $apiarray['order_items'][$i]['price']     =   $product->price;
//                 $apiarray['order_items'][$i]['allergies'] =   $allergies;
                 $apiarray['order_items'][$i]['allergies'] =   '';
                 $apiarray['order_items'][$i]['image']     =   'https://mysushi.ph/img/products/'.$product->desktop_pic;
                 $i++;
             }

         //saving client info starts
             $apiarray['client']['name']                 =   $user->first_name.' '.$user->last_name;
             $apiarray['client']['email']                =   $user->email;
             $apiarray['client']['phone_no']             =   $user->phone;
             if(empty($user->google_user_id) && empty($user->fb_user_id))
             {
                 if(!empty($user->profile_pic))
                 {
                     $user->profile_pic = 'https://mysushi.ph/'.$user->profile_pic;
                 }
             }
             $apiarray['client']['image']                =   $user->profile_pic;
             $apiarray['client']['total_orders']         =   $user->no_of_orders;
             $apiarray['client']['total_spend']          =   $user->total_spent;
             $apiarray['client']['source']               =   'MySushi';

//             dd($apiarray);
         //saving client info ends

//          dd($this->dfg_api_baseurl);
//             $response = Http::post($this->dfg_api_baseurl.'/api/insert_orders', [
//                 'apiarray' => $apiarray,
//             ]);
//
//         dd($response->body());
//
//        $client = new \GuzzleHttp\Client();
//        $res = $client->request('POST', 'http://127.0.0.2:8000/api/insert_orders', [
//            'apiarray' => $apiarray,
//        ]);
//        echo $res->getStatusCode();
//// "200"
//        echo $res->getHeader('content-type')[0];
// 'application/json; charset=utf8'
//        dd(json_encode($apiarray));
//        $apiarray = json_encode($apiarray, true);
//        $apiarray = json_encode( array("apiarray"=> $apiarray));
        $curl = curl_init();

        //call login function of affiliate to get jwt token
        $headers = [
            'Content-Type: application/json',
        ];
//        dd($this->dfg_api_baseurl."/api/insert_orders_mysushi");
        curl_setopt_array($curl, array(
            CURLOPT_URL => $this->dfg_api_baseurl."/api/insert_orders_mysushi",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 600,
            CURLOPT_POSTFIELDS => json_encode($apiarray),
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_HTTPHEADER => $headers,
            CURLOPT_CUSTOMREQUEST => "POST"

        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);
        curl_close($curl);

        if ($err) {
            $response = $err;
        }
        else
        {
//            $response    =   json_decode($response);
        }
//        echo $response;die;
//        echo "<pre>";print_r($response);die;

        // Call api to insert record in DFG database ends

//        $paypal_data = json_decode(@$request->paypal, true);
//        if (count(@$paypal_data) != 0) {
//            $paypal = new Paypal;
//            $paypal->order_id = $order->id;
//            $paypal->cart = $paypal_data['cart'];
//            $paypal->create_time = date("Y-m-d H:i:s", strtotime($paypal_data['create_time']));
//            $paypal->paypal_id = $paypal_data['id'];
//            $paypal->intent = $paypal_data['intent'];
//            $paypal->payer = json_encode($paypal_data['payer']);
//            $paypal->state = $paypal_data['state'];
//            $paypal->transactions = json_encode($paypal_data['transactions']);
//            $paypal->save();
//        }



//        $arrResult = [];
//        $arrResult["order_id"] = $order->id;
//
//        echo json_encode($arrResult);
        return [
            'order_id' => $orderID,
            'status' => $status
        ];
/* ------- TALHA CODE ENDS HERE ---------- */
    }
/* ------- TALHA CODE STARTS HERE ---------- */
    public function savePaypalInfo(Request $request){
        $paypal_data = json_decode(@$request->paypal, true);
        if (count(@$paypal_data) != 0) {
            Order::find($request->orderID)->update(['is_paid' => '1']);
            $paypal = new Paypal;
            $paypal->order_id = $request->orderID;
            $paypal->cart = $paypal_data['cart'];
            $paypal->create_time = date("Y-m-d H:i:s", strtotime($paypal_data['create_time']));
            $paypal->paypal_id = $paypal_data['id'];
            $paypal->intent = $paypal_data['intent'];
            $paypal->payer = json_encode($paypal_data['payer']);
            $paypal->state = $paypal_data['state'];
            $paypal->transactions = json_encode($paypal_data['transactions']);
            $paypal->save();

            return [
                'status' => 200
            ];
        }
    }
/* ------- TALHA CODE ENDS HERE ---------- */

    /**
     * Get total orders count
     *
     * @return JSON      orders count data
     */
    public function totalOrdersStats()
    {
        $currStartMonth = Carbon::today()->startOfMonth();
        $currEndMonth = Carbon::today()->endOfMonth();
        $prevStartMonth = Carbon::now()->subMonth()->startOfMonth();
        $prevEndMonth = Carbon::now()->subMonth()->endOfMonth();

        $ordersCount = Order::get()->count();
        $ordersCountThisMonth = Order::whereBetween('created_at', [$currStartMonth, $currEndMonth])->count();
        $ordersCountPrevMonth = Order::whereBetween('created_at', [$prevStartMonth, $prevEndMonth])->count();
        $ordersTotalAmount = Order::sum('total_amount');
        $ordersTotalAmountThisMonth = Order::whereBetween('created_at', [$currStartMonth, $currEndMonth])->sum('total_amount');
        $ordersTotalAmountPrevMonth = Order::whereBetween('created_at', [$prevStartMonth, $prevEndMonth])->sum('total_amount');

        $ordersData = new \stdClass();
        $ordersData->count = $ordersCount;
        $ordersData->count_this_month = $ordersCountThisMonth;
        $ordersData->count_prev_month = $ordersCountPrevMonth;
        $ordersData->curr_percentage = ($ordersCountThisMonth === 0 && $ordersCountThisMonth === 0) ? 0 : $ordersCountThisMonth / $ordersCount * 100;
        $ordersData->prev_percentage = ($ordersCountPrevMonth === 0 && $ordersCountPrevMonth === 0) ? 0 : $ordersCountPrevMonth / $ordersCount * 100;
        $ordersData->total_amount = $ordersTotalAmount;
        $ordersData->curr_total_amount = $ordersTotalAmountThisMonth;
        $ordersData->prev_total_amount = $ordersTotalAmountPrevMonth;
        $ordersData->curr_total_amount_percentage = ($ordersTotalAmountThisMonth === 0 && $ordersTotalAmount === 0) ? 0 : $ordersTotalAmountThisMonth / $ordersTotalAmount * 100;
        $ordersData->prev_total_amount_percentage = ($ordersTotalAmountPrevMonth === 0 && $ordersTotalAmount === 0) ? 0 : $ordersTotalAmountPrevMonth / $ordersTotalAmount * 100;

        return response()->json($ordersData, 200);
    }

    /**
     * Get monthly total orders count
     *
     * @return JSON      orders count data
     */
    public function monthlyOrdersStats()
    {
        $orders = DB::table('orders')
            ->selectRaw('DATE_FORMAT(created_at, "%b") AS month, COUNT(*) AS total_orders, SUM(total_amount) AS total_amount')
            ->where('created_at', '>', '"' . Carbon::now()->subMonths(5)->startOfMonth()->format('Y-m-d') . '"')
            ->groupBy(DB::raw('DATE_FORMAT(created_at, "%b")'))
            ->orderBy('id', 'desc')
            ->get();

        return response()->json($orders, 200);
    }

    public function myorderlist(Request $request)
    {
        $orders = new Order;
        $orders_data = $orders::select('*', DB::raw('DATE_FORMAT(created_at, "%d %b, %Y at  %H:%i:%s %p") as date'))
            ->where('client_id', $request->id)
            ->orderBy('id', 'desc')->get();
        return $orders_data;
    }
    public function countOrders(Request $request){
        $orders = new Order;
        $orders_data = $orders::count();
        return $orders_data;
    }
}
