<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Http\Requests\SettingsRequest;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use App\Models\Setting;
use Image;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;
class SettingsController extends Controller
{
    /**
     * Get settings data
     *
     * @return  JSON  settings data
     */
    public function index()
    {
        $settings = Setting::first();
        $settings = json_decode($settings, true);
        $date_and_time['server_date_and_time'] =date("Y-m-d H:i:s");
        $data = array_merge($settings, $date_and_time);
        return response()->json($data, 200);
    }

    /**
     * Update settings
     *
     * @param   SettingsRequest  $request  request data
     *
     * @return  JSON                       settings update data with status and message
     */
    public function update(SettingsRequest $request) {
        $settingsUpdate = Setting::first();
        $request->settings = json_decode($request->settings);

        if (empty($settingsUpdate)) {
            try {
                DB::beginTransaction();

                $settings = new Setting();
                $settings->store_lat = $request->settings->store_lat;
                $settings->store_long = $request->settings->store_long;
                $settings->store_distance = $request->settings->store_distance;
                $settings->country = $request->settings->country;
                $settings->affiliate_apis = $request->settings->affiliate_apis;
                $settings->min_order_amount = number_format($request->settings->min_order_amount, 2);
                $settings->delivery_fee = number_format($request->settings->delivery_fee, 2);
                $settings->min_commission = number_format($request->settings->min_commission, 2);
                $settings->max_commission = number_format($request->settings->max_commission, 2);
                $settings->commission_change_limit = $request->settings->commission_change_limit;
                $settings->open_time = gmdate('H:i:s', strtotime($request->settings->open_time));
                $settings->close_time = gmdate('H:i:s', strtotime($request->settings->close_time));
                $settings->deactive_dates = $request->settings->deactive_dates;
                $settings->store_status = $request->settings->store_status ? "1" : "0";

                $home_background ='';
                if ($request->hasFile('home_background') && !empty($request->file('home_background'))) {
                  @unlink('img/home_background.png');
                  $homeimg = $request->file('home_background');
                  $home_background = 'home_background.png';
                  // Image::make($request->file('home_background')->getRealPath())->save($home_background);
                  Storage::disk('frontend')->put($home_background,  File::get($homeimg));

                  $settings->home_background = (!empty($home_background)) ? 'img/'.$home_background : null;
                }
                $mobile_background ='';
                if ($request->hasFile('mobile_background') && !empty($request->file('mobile_background'))) {
                  @unlink('img/mobile_background.png');
                  $mobimg = $request->file('mobile_background');
                  $mobile_background = 'mobile_background.png';
                  /*Image::make($request->file('mobile_background')->getRealPath())->save($mobile_background);*/
                  Storage::disk('frontend')->put($mobile_background,  File::get($mobimg));

                  $settings->mobile_background = (!empty($mobile_background)) ? 'img/'.$mobile_background : null;
                }

                if ($settings->save()) {
                    DB::commit();

                    return response()->json([
                        'success' => true,
                        'data' => $settings
                    ], 200);
                } else {
                    DB::rollback();

                    return response()->json([
                        'success' => false,
                        'message' => 'Error creating settings.'
                    ], 500);
                }
            } catch(QueryException $qe) {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => $qe->getMessage()
                ], 500);
            } catch(\Exception $e) {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => $e->getMessage()
                ], 500);
            }
        } else {
            try {
                DB::beginTransaction();

                $settingsUpdate->store_lat = $request->settings->store_lat;
                $settingsUpdate->store_long = $request->settings->store_long;
                $settingsUpdate->store_distance = $request->settings->store_distance;
                $settingsUpdate->country = $request->settings->country;
                $settingsUpdate->min_order_amount = number_format($request->settings->min_order_amount, 2);
                $settingsUpdate->delivery_fee = number_format($request->settings->delivery_fee, 2);
                $settingsUpdate->min_commission = number_format($request->settings->min_commission, 2);
                $settingsUpdate->max_commission = number_format($request->settings->max_commission, 2);
                $settingsUpdate->commission_change_limit = $request->settings->commission_change_limit;
                $settingsUpdate->affiliate_apis = $request->settings->affiliate_apis;
                $settingsUpdate->open_time = gmdate('H:i:s', strtotime($request->settings->open_time));
                $settingsUpdate->close_time = gmdate('H:i:s', strtotime($request->settings->close_time));
                $settingsUpdate->deactive_dates = $request->settings->deactive_dates;
                $settingsUpdate->store_status = $request->settings->store_status ? "1" : "0";

                $home_background ='';
                if ($request->hasFile('home_background') && !empty($request->file('home_background'))) {

                  @unlink('img/home_background.png');
                  $homeimg = $request->file('home_background');
                  $home_background = 'home_background.png';
                 // Image::make($request->file('home_background')->getRealPath())->save($home_background);
                  Storage::disk('frontend')->put($home_background,  File::get($homeimg));
                  $settingsUpdate->home_background = (!empty($home_background)) ? 'img/'.$home_background : null;
                }

              $mobile_background ='';
              if ($request->hasFile('mobile_background') && !empty($request->file('mobile_background'))) {

                @unlink('img/mobile_background.png');
                $mobimg = $request->file('mobile_background');
                $mobile_background = 'mobile_background.png';
                /*Image::make($request->file('mobile_background')->getRealPath())->save($mobile_background);*/
                Storage::disk('frontend')->put($mobile_background,  File::get($mobimg));
                $settingsUpdate->mobile_background = (!empty($mobile_background)) ? 'img/'.$mobile_background : null;
              }




                if ($settingsUpdate->update()) {
                    DB::commit();

                    return response()->json([
                        'success' => true,
                        'data' => $settingsUpdate
                    ], 200);
                } else {
                    DB::rollback();

                    return response()->json([
                        'success' => false,
                        'message' => 'Error updating settings.'
                    ], 500);
                }
            } catch(QueryException $qe) {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => $qe->getMessage()
                ], 500);
            } catch(\Exception $e) {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => $e->getMessage()
                ], 500);
            }
        }
    }
}
